/**
 * Memory Analyser  version 1.00
 * Copyright (c) 2018, Tycho Veltmeijer.
 * All rights reserved.
**/

#include <stdio.h>
#include "memory_analyser.h"
#include "Filter_Firefox.h"
#include "Filter_Safari.h"
#include "Filter_IE.h"
#include "Filter_Dump.h"
#include "Filter_General.h"
#include "list.h";

#define ARGUMENT_FLAG_HAS_VALUE 0x1
#define ARGUMENT_FLAG_HAS_OPTIONAL_VALUE 0x2

#define AMOUNT_OF_ARGUMENTS 16

char* arguments[AMOUNT_OF_ARGUMENTS] = {"pid", "file", "l", "fsu", "fpa", "fca", "gua", "guu", "sd", "bd", "ssa", "sra", "sca", "iua", "iuu", "ica"};

struct
{   InitiateFilter initiate;
    MEMFUNC			apply;
    Destroy			destroy;
} argumentsFunctions[AMOUNT_OF_ARGUMENTS] =
{
	{0, 0, 0},	//pid
	{0, 0, 0},	//file
	{0, 0, 0},	//l
	{(InitiateFilter) InitiateFirefoxFilter1, (MEMFUNC) ApplyFirefoxFilter1, (Destroy) DestroyFirefoxFilter1}, //fsu
	{(InitiateFilter) InitiateFirefoxFilter2, (MEMFUNC) ApplyFirefoxFilter2, (Destroy) DestroyFirefoxFilter2}, //fpa
	{(InitiateFilter) InitiateFirefoxFilter3, (MEMFUNC) ApplyFirefoxFilter3, (Destroy) DestroyFirefoxFilter3}, //fca
	{(InitiateFilter) InitiateFilterGeneralASCII, (MEMFUNC) ApplyFilterGeneralASCII, (Destroy) DestroyFilterGeneralASCII}, //gua
	{(InitiateFilter) InitiateFilterGeneralUNICODE, (MEMFUNC) ApplyFilterGeneralUNICODE, (Destroy) DestroyFilterGeneralUNICODE},	//guu
	{(InitiateFilter) InitiateDumpSmall, (MEMFUNC) WriteMemoryDump, (Destroy) DestroyDump},	//sd
	{(InitiateFilter) InitiateDumpBig, (MEMFUNC) WriteMemoryDump, (Destroy) DestroyDump},	//bd
	{(InitiateFilter) InitiateSafariFilter2, (MEMFUNC) ApplySafariFilter2, (Destroy) DestroySafariFilter2}, //ssa
	{(InitiateFilter) InitiateSafariFilter1, (MEMFUNC) ApplySafariFilter1, (Destroy) DestroySafariFilter1}, //sra
	{(InitiateFilter) InitiateSafariFilter3, (MEMFUNC) ApplySafariFilter3, (Destroy) DestroySafariFilter3}, //sca
	{(InitiateFilter) InitiateIEFilter1, (MEMFUNC) ApplyIEFilter1, (Destroy) DestroyIEFilter1}, //iua
	{(InitiateFilter) InitiateIEFilter2, (MEMFUNC) ApplyIEFilter2, (Destroy) DestroyIEFilter2}, //iuu
	{(InitiateFilter) InitiateIEFilter3, (MEMFUNC) ApplyIEFilter3, (Destroy) DestroyIEFilter3}  //ica
};

int argumentsFlags[AMOUNT_OF_ARGUMENTS] = {
	ARGUMENT_FLAG_HAS_VALUE,  //pid
	ARGUMENT_FLAG_HAS_VALUE,  //file 
	ARGUMENT_FLAG_HAS_VALUE,  //location
	ARGUMENT_FLAG_HAS_OPTIONAL_VALUE, //Firefox String URL Unicode
	ARGUMENT_FLAG_HAS_OPTIONAL_VALUE, //Firefox Private session URL ASCII search
	ARGUMENT_FLAG_HAS_OPTIONAL_VALUE, //Firefox Cookie Ascii search
	ARGUMENT_FLAG_HAS_OPTIONAL_VALUE, //General URL ASCII search
	ARGUMENT_FLAG_HAS_OPTIONAL_VALUE, //General URL UNICODE search
	0, //small dump
	0, //big dump
	ARGUMENT_FLAG_HAS_OPTIONAL_VALUE, //Safari String ASCII search
	ARGUMENT_FLAG_HAS_OPTIONAL_VALUE, //Safari Recursive String ASCII search
	ARGUMENT_FLAG_HAS_OPTIONAL_VALUE, //Safari Cookie Ascii search
	ARGUMENT_FLAG_HAS_OPTIONAL_VALUE, //Internet Explorer URL search (ASCI)
	ARGUMENT_FLAG_HAS_OPTIONAL_VALUE, //Internet Explorer URL search (UNICODE)
	ARGUMENT_FLAG_HAS_OPTIONAL_VALUE //Internet Explorer Cookie search (ASCII)
};
char* argumentValue[AMOUNT_OF_ARGUMENTS] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

main( int argc, const char* argv[])
{
	DWORD processPid;	//PID of currently handeld of process
	HANDLE process;		//Handle to process
	HANDLE inputFile;	//FileHandle if the input is a memory dump
	int i, j;	
	int analyse_type;	//ANALYSE_FILE or ANALYSE_MEMORY
	int exitCode;		//Returnvalue of a process when closed
	char* openProcess="firefox.exe";	//name of process to be processed, default: firefox.exe
	char* defaultOpenProcess;
	char* location="C:\\temp\\dump\\";	//Output location, default: C:\temp\dump\ 
	char* processName;	//path of this progam, is used to cal itself to process multiple PID's
	char* commandLine;	//commandline to call itself
	char* newLocation;	//location + the current being processed PID
	STARTUPINFO startupInfo = {0};
	LPPROCESS_INFORMATION proc = {0};
	MEMFUNC_STRUCT* mem_functions;		//structure with all the functions that need to called to analyse the memory
	MEMFUNC_STRUCT* tmp_mem_functions;	//temperary structure to fill mem_functions

	/*zero everything*/
	processPid = 0;
	inputFile = 0;
	process = 0;
	defaultOpenProcess = openProcess;
	/*fill in size member STARTUPINFO*/
	startupInfo.cb = sizeof(STARTUPINFO);

	/*Request pad of this program*/
	processName = (char*) malloc(MAX_PATH+1);
	GetModuleFileName(0, processName, MAX_PATH);

	/*requst memory*/
	commandLine = (char*) malloc(MAX_PATH *2 + 2);
	newLocation = (char*) malloc(MAX_PATH+1);
	proc = (LPPROCESS_INFORMATION) malloc(sizeof(PPROCESS_INFORMATION));

	/*Process arguments, set argumentValue to -1 if the argument has no sub-arguments and 0 if the argument is not pressent.*/
	for(i=1; i<argc; i++) {
		if(argv[i][0]=='-') {
			for(j=0; j<AMOUNT_OF_ARGUMENTS; j++) {
				if(stricmp((char*) argv[i]+1, arguments[j])==0) {
					if((argumentsFlags[j]&ARGUMENT_FLAG_HAS_VALUE && argc!=i)
					|| (argumentsFlags[j]&ARGUMENT_FLAG_HAS_OPTIONAL_VALUE && argc-1!=i && argv[i+1][0]!='-')) {
						argumentValue[j] = (char*) argv[i+1];
						i++;
						break;
					} else if(!(argumentsFlags[j]&ARGUMENT_FLAG_HAS_VALUE)) {
						argumentValue[j] = (char*) -1;
					}
				}
			}
		} else {
			openProcess = (char*)argv[i];
		}
	}

	if(!(argumentValue[ARGUMENT_PID]!=0 && defaultOpenProcess!=openProcess))
		printf(
			"-----------------------------------------------------------------------\n"
			"|                     Memory Analyser  version 1.00                    |\n"
			"|                 Copyright (c) 2018, Tycho Veltmeijer.                |\n"
			"|                         All rights reserved.                         |\n"
			"-----------------------------------------------------------------------\n\n");

	if(argumentValue[ARGUMENT_FILE] == 0) {
		/*Convert PIDfrom the arguments to an integer*/
		if(argumentValue[ARGUMENT_PID]!=0) {
			processPid = strtol(argumentValue[ARGUMENT_PID], NULL, 10);
			openProcess = 0;
		}

		/*Get PID from processname (from the arguments*/
		if(openProcess!=0) {
			processPid = FindProcess(openProcess, 0);
			if(processPid==0) {
				MessageBox(0, "Het opgegeven proces draaid niet.","Fout",0x10);
				return 0;
			}

			/*If there are multipple PID's with the samen processname, call this program for every process*/
			if(FindProcess(openProcess, processPid) != 0) {
				/*Fill in the default location*/
				if(argumentValue[ARGUMENT_LOCATION]==0)
					argumentValue[ARGUMENT_LOCATION] = location;
				do {
					/*Create new outputfolder with the PID as name*/
					printf("Processing PID %d \n", processPid);
					sprintf(newLocation, "%s/%d", argumentValue[ARGUMENT_LOCATION], processPid);
					CreateDirectory(newLocation, 0); 

					/*Create new commandline*/
					sprintf(commandLine, "%s -pid %d -l \"%s\"", GetCommandLine(), processPid, newLocation);

					/*Start the process*/
					CreateProcess(processName, commandLine, 0, 0, 0, 0, 0, 0, &startupInfo, proc);

					/*Wait until the process is closed*/
					exitCode=STILL_ACTIVE;
					while(exitCode==STILL_ACTIVE) {
						Sleep(100);
						GetExitCodeProcess(proc->hProcess, (LPDWORD) &exitCode);
					}

					/*Het next process in line*/
					processPid = FindProcess(openProcess, processPid);
					if(processPid==0) {
						return 0;
					}

					printf("\n");
				} while(1);
			}
		}

		/*If only one PID left, try to get full acces to it*/
		process = OpenProcess(PROCESS_ALL_ACCESS, 0, processPid);
		if(process==NULL) {
			MessageBox(0, "Kon het process niet openen.","Fout",0x10);
			return 0;
		}
	} else {
		/*Open the input file.*/
		inputFile = CreateFile(argumentValue[ARGUMENT_FILE],
			GENERIC_READ,
			FILE_SHARE_READ,
			NULL,
			OPEN_EXISTING,
			NULL,
			NULL
		);

		/*Error if this failed*/
		if(inputFile==INVALID_HANDLE_VALUE) {
			showLastError();
			exit(0);
		}

	}

	/*If no filter is pressent in the commandline, enable the Firefox filters*/
	if(argumentValue[ARGUMENT_FSU] == (char*) 0
	&& argumentValue[ARGUMENT_FPA] == (char*) 0
	&& argumentValue[ARGUMENT_FCA] == (char*) 0
	&& argumentValue[ARGUMENT_GUA] == (char*) 0
	&& argumentValue[ARGUMENT_GUU] == (char*) 0
	&& argumentValue[ARGUMENT_SD] == (char*) 0
	&& argumentValue[ARGUMENT_BD] == (char*) 0
	&& argumentValue[ARGUMENT_SSA] == (char*) 0
	&& argumentValue[ARGUMENT_SRA] == (char*) 0
	&& argumentValue[ARGUMENT_SCA] == (char*) 0
	&& argumentValue[ARGUMENT_IUA] == (char*) 0
	&& argumentValue[ARGUMENT_IUU] == (char*) 0
	&& argumentValue[ARGUMENT_ICA] == (char*) 0) {
		argumentValue[ARGUMENT_FSU] = (char*) -1;
		argumentValue[ARGUMENT_FPA] = (char*) -1;
		argumentValue[ARGUMENT_FCA] = (char*) -1;
		argumentValue[ARGUMENT_GUA] = (char*) -1;
		argumentValue[ARGUMENT_GUU] = (char*) -1;
		argumentValue[ARGUMENT_SD] = (char*) -1;
	}

	/*Create a structure thatwill contain all the functions that need to be called during the memory analyse*/
	mem_functions = (MEMFUNC_STRUCT*) malloc(sizeof(MEMFUNC_STRUCT));

	/*Fill in the default location*/
	if(argumentValue[ARGUMENT_LOCATION]==0)
		argumentValue[ARGUMENT_LOCATION] = location;
	/*Set first function to zero, so it's clear this is not filled in yet*/
	tmp_mem_functions = mem_functions;
	tmp_mem_functions->func = 0;

	/*Memory or file analyse?*/
	if(inputFile==(HANDLE) 0)
		analyse_type = ANALYSE_MEMORY;
	else
		analyse_type = ANALYSE_FILE;

	/*Fill mem_functions with all the filters that need to be used*/
	for(i=0; i<AMOUNT_OF_ARGUMENTS; i++) {
		if(argumentsFunctions[i].initiate !=0 && argumentValue[i]!=0) {
			if(tmp_mem_functions->func!=0) {
				tmp_mem_functions->next = (MEMFUNC_STRUCT*) malloc(sizeof(MEMFUNC_STRUCT));
				tmp_mem_functions = tmp_mem_functions->next;
			}
			tmp_mem_functions->param = argumentsFunctions[i].initiate(argumentValue[ARGUMENT_LOCATION], argumentValue[i], analyse_type);
			tmp_mem_functions->func = argumentsFunctions[i].apply;
			tmp_mem_functions->funcDestroy = argumentsFunctions[i].destroy;
			tmp_mem_functions->flags = 0;
		}
	}

	/*Set next to zero to indicate the last filter*/
	tmp_mem_functions->next = 0;
	
	/*Start the search*/
	if(inputFile==(HANDLE) 0) {
		searchMemory(process, mem_functions);
	} else {
		searchFile(inputFile, mem_functions);
	}

	/*Release memory (and somtimes some finishing jobs like IE cookies filter).*/
	tmp_mem_functions = mem_functions;
	while(tmp_mem_functions!=0) {
		tmp_mem_functions->funcDestroy((MEMFUNC_STRUCT*) tmp_mem_functions->param);
		tmp_mem_functions = tmp_mem_functions->next;
	}

	/*Done :)*/
	printf("done");
	return 0;
}

void showLastError() {
	showError(GetLastError());
}

void showError(int code) {
	char* lpBuffer;

	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER+FORMAT_MESSAGE_FROM_SYSTEM, 0, code, LANG_NEUTRAL, (char*) &lpBuffer, 0, 0);
	MessageBox(0, lpBuffer, "Error", MB_ICONERROR+MB_OK);
	LocalFree(lpBuffer);
} 

/*The filter are made for analysing virtual memory. To let them apply on memory dumps, MEMORY_BASIC_INFORMATION will be filled with fake information.
 Since a memory dump can be quite large, it cannot be loaded into memory all at once, it has to be split up.
 The problem with this is that you find the beginning of a URL, but cannot find the end, since it's exactly where the file has been split up.
 The answer to this problem is 'overflow': the loaded data will look like this: 'buffer (511mb) | overflow (1mb)'
 The overflow part may not be analysed, however, if something is found in the buffer part, the overflow part can be used to recover the found data without haveing the wait for the next round.
*/
int searchFile(HANDLE inputFile, MEMFUNC_STRUCT* memf) {
	int retv;
	char* fileContents;
	long long fileSize=0;
	long long filepos =0;
	long readSize=0;
	LARGE_INTEGER currentPos;
	MEMFUNC_STRUCT* tmp_memf;
	MEMORY_BASIC_INFORMATION memInfo={0};
	GetFileSizeEx(inputFile, (PLARGE_INTEGER) &fileSize);

	/*request buffer for the file*/
	if(fileSize + default_empty_overflow_size < default_buffer_size) {
		fileContents = (char*) malloc(fileSize + default_empty_overflow_size);
		if(fileContents == 0) {
			showLastError();
			exit(0);
		}

	} else {
		fileContents = (char*) malloc(default_buffer_size);
		if(fileContents == 0) {
			showLastError();
			exit(0);
		}
	}

	filepos = fileSize;

	/*Read the file*/
	while(filepos!=0) {
		/*Are we at the end of the file?*/
		if(filepos + default_empty_overflow_size > default_buffer_size)
			readSize = default_buffer_size - default_empty_overflow_size;
		else
			readSize = filepos;

		printf("Reading at position %lld / %lld Readingsize: %d... (%-3f%%)\n", fileSize - filepos, fileSize, readSize, ((((float)fileSize - (float)filepos)/ (float)fileSize) *100));

		/*Fill memInfo with fake info .*/
		ReadFile(inputFile, fileContents, readSize, (LPDWORD) &retv, 0);
		VirtualQuery((LPCVOID) fileContents, &memInfo, sizeof(MEMORY_BASIC_INFORMATION));
		memInfo.BaseAddress = (PVOID) (fileSize - filepos);

		/*Take care of the overflow parameter*/
		if(filepos + default_empty_overflow_size > default_buffer_size) {
			memInfo.RegionSize = readSize - default_buffer_overflow_size;
			filepos -= retv - default_buffer_overflow_size;			
		} else {
			memInfo.RegionSize = readSize;
			filepos -= readSize;			
		}
		currentPos.QuadPart = fileSize - filepos;
		if(SetFilePointerEx(inputFile, currentPos, 0, FILE_BEGIN)==0) 
			showLastError();

		/*request current possion in the file*/
		currentPos.QuadPart = 0;
		SetFilePointerEx(inputFile, currentPos, &currentPos, FILE_CURRENT);

		filepos = fileSize - currentPos.QuadPart;

		/*call the functions to process this peace of memory*/
		tmp_memf = memf;
		printf("Processing memory...\n");
		while(tmp_memf!=0) { /*Call all functions in MEMFUNC_STRUCT*/
			retv = tmp_memf->func(tmp_memf->param, 0, &memInfo, fileContents, readSize);

			if(tmp_memf->flags&MS_RETURN_ON_ZERO && retv==0)
				return retv;

			if(tmp_memf->flags&MS_RETURN_NOT_ZERO && retv!=0)
				return retv;

			tmp_memf = tmp_memf->next;
		}
	}

	free(fileContents);
}

//Read the virtual memory and call the functions in MEMFUNC_STRUCT to analyse it.
int searchMemory(HANDLE process, MEMFUNC_STRUCT* memf) {
	memlong memLocation;
	int retv;
	int oldProtect=-1;
	char* buff=0;
	MEMORY_BASIC_INFORMATION memInfo;
	MEMFUNC_STRUCT* tmp_memf;

	for (memLocation = min_mem_location; memLocation < max_mem_location; memLocation++) {
		/*request info about the memory: size and rights */
		retv = VirtualQueryEx(process, (LPCVOID) memLocation, &memInfo, sizeof(MEMORY_BASIC_INFORMATION));
		if(retv!=0 && retv==sizeof(MEMORY_BASIC_INFORMATION)) { 
			oldProtect=-1;

			if(memInfo.State!=MEM_FREE && memInfo.State!=MEM_RESERVE) { //Is the memory location used by the process?
				/*Is memory readable? if not assign readrights to it*/
				retv = memInfo.Protect;
				retv = retv & PAGE_NOACCESS;
				retv |= (memInfo.Protect) & PAGE_EXECUTE;
				
				if(retv != 0) { //give readrights
					retv = (int) VirtualProtectEx(process, (LPVOID) memInfo.BaseAddress, memInfo.RegionSize, PAGE_EXECUTE_READWRITE, (PDWORD) &oldProtect);
				}
				
				/*Read the virtual memory*/
				buff = (char*) malloc(memInfo.RegionSize);
				retv = ReadProcessMemory(process, memInfo.BaseAddress, buff, memInfo.RegionSize, NULL);

				/*return old memory rights*/
				if(oldProtect != -1)
					retv = (int) VirtualProtectEx(process, (LPVOID) memInfo.BaseAddress, memInfo.RegionSize, oldProtect, (PDWORD) &oldProtect);
			} else {
				buff = 0;
			}

			tmp_memf = memf;
			while(tmp_memf!=0) { /*Call all functions in MEMFUNC_STRUCT*/
				retv = tmp_memf->func(tmp_memf->param, process, &memInfo, buff, memInfo.RegionSize);

				if(tmp_memf->flags&MS_RETURN_ON_ZERO && retv==0)
					return retv;

				if(tmp_memf->flags&MS_RETURN_NOT_ZERO && retv!=0)
					return retv;

				tmp_memf = tmp_memf->next;
			}

			//release buffer
			if(buff!=0)
				free(buff);

			//Readout next part of the memory
			if((memlong) memInfo.BaseAddress + (memlong) memInfo.RegionSize - 1 == memLocation)
				break;
			else
				memLocation = (memlong) memInfo.BaseAddress + (memlong) memInfo.RegionSize - 1;
		} else {
			break;
		}
	}
}

/*Call CreateMemoryDump and write string "" 
to the file to be an UNICODE-text file*/
DUMP_SESSION* CreateUNICODEMemoryDump(char* outputDir, char* outputFile, char* optionalOutputFile, int flags) {
	int writtenBytes=0;
	DUMP_SESSION* newdump;

	newdump = CreateMemoryDump(outputDir, outputFile, optionalOutputFile, flags);
	WriteFile(newdump->dumpFile,"", 2, (LPDWORD) &writtenBytes, NULL);
	if(newdump->optionalDumpFile) WriteFile(newdump->optionalDumpFile,"", 2, (LPDWORD) &writtenBytes, NULL);

	return newdump;
}

//Create a DUMP_SESSION structure
DUMP_SESSION* CreateMemoryDump(char* outputDir, char* outputFile, char* optionalOutputFile, int flags) {
	DUMP_SESSION* newdump;
	char* outPath=0;
	char* openFile=0;

	if(outputDir!=0) {
		outPath = (char*) malloc(strlen(outputDir) + strlen(outputFile) + 10);
		strcpy(outPath, outputDir);
		if(outputDir[strlen(outputDir)-1] != '/' && outputDir[strlen(outputDir)-1] != '\\')
			strcat(outPath, "\\");
		strcat(outPath, outputFile);
		openFile = outPath;
	} else {
		openFile = outputFile;
	}


	newdump = (DUMP_SESSION*) malloc(sizeof(DUMP_SESSION));
	newdump->dumpFile = CreateFile(openFile,
		(GENERIC_READ | GENERIC_WRITE),
		0,
		NULL,
		CREATE_ALWAYS,
		0,
		0);

	if(outPath!=0)
		free(outPath);

	if(optionalOutputFile!=0) {
		if(outputDir!=0) {
			outPath = (char*) malloc(strlen(outputDir) + strlen(optionalOutputFile) + 10);
			strcpy(outPath, outputDir);
			if(outputDir[strlen(outputDir)-1] != '/' && outputDir[strlen(outputDir)-1] != '\\')
				strcat(outPath, "\\");
			strcat(outPath, optionalOutputFile);
			openFile = outPath;
		} else {
			openFile = outputFile;
		}
		newdump->optionalDumpFile = CreateFile(openFile,
			(GENERIC_READ | GENERIC_WRITE),
			0,
			NULL,
			CREATE_ALWAYS,
			0,
			0);

		if(outPath!=0)
			free(outPath);
	} else {
		newdump->optionalDumpFile=0;
	}

	newdump->flags = flags;
	newdump->string_location = (char*) malloc(1000);
	newdump->unlocated_size = 0;
	newdump->zerobuf = (char*) malloc(1000);
	memset(newdump->zerobuf, 0, 1000);

	return newdump;
}

//delete DUMP_SESSION structure
void Destroy_MemoryDump(DUMP_SESSION* dumpsess) {
	CloseHandle(dumpsess->dumpFile);
	if(dumpsess->optionalDumpFile!=0) CloseHandle(dumpsess->optionalDumpFile);
	free(dumpsess->string_location);
	free(dumpsess->zerobuf);
	free(dumpsess);
}

/* function FindProcess
	returns the PID of a process corresponding with the processname

	name the programname to find the PID from
	nextProcess last returned PID to get the next running process
*/
DWORD FindProcess(char *name, DWORD nextProcess) {
	PROCESSENTRY32 procEntry;
	int foundPreviousProcess=0;

	HANDLE hSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0); //a handle to a snapshot of processes
	if (hSnap == INVALID_HANDLE_VALUE) { 
		return 0 ;
	}

	procEntry.dwSize = sizeof(PROCESSENTRY32);

	if (!Process32First(hSnap, &procEntry))	{ //Get first process in the snapshot
		return 0 ;
	}

	do {
		if (_stricmp(procEntry.szExeFile, name) == 0) { //processname equal to the one we are looking for?
			if(nextProcess==0
			|| (
			nextProcess!=0
			&& foundPreviousProcess==1
			)) {
				CloseHandle(hSnap);
				return(procEntry.th32ProcessID);

			} else {
				if(nextProcess==procEntry.th32ProcessID)
					foundPreviousProcess = 1;
			}
		}
	} while (Process32Next(hSnap, &procEntry)); //Get next process in the snapshot

	/*the process we are looking for is not running*/
	CloseHandle(hSnap); 
	return(0);
}